/*
/ Projektname : LCD-I2C-Modul
/ Dateiname   : I2C-Display.c
/ Version     : 1.0.0
/ Datum       : 18.01.2014
/ Compiler    : WinAVR 20100110 libc 1_7_1
/ Autor       : Johannes Kretzmann
/ Copyright   : johannes.kretzmann@projektsammlung.de
/               www.projektsammlung.de
*/

 
#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>
#include "I2C.h"
#include "I2C-Display.h"


/* -------------------------------------------------------------------------------------
   Initialisierungsroutine fr das Display 
   Parametereinstellungen: 4 Bit Modus aktivieren                   0x02
                           2 Zeilen, Zeichen 5x7                    0x28
                           Display ein, Cursor aus, kein Blinken    0x0C
                           Speicher inkrementieren, kein Scrollen   0x06
--------------------------------------------------------------------------------------*/ 
void InitDisplay(void)
{
   _delay_us(15000);                                                    // --- Warte 15 ms nach Power-On ---

   // Setze Bit fr Hintergrundbeleuchtung auf ein
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_BACKLIGHT);
   
   // Setze RS auf low um Daten ins Steuerregister zu schreiben
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_RS);                 // Setze RS auf 0; Steuerregister
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_RW);                 // Zum Schreiben RW-Pin auf 0 setzen
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
 

   // Initialisierung erfolgt noch mit 8 Bit
   // Function set 0x03;
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_DATA4);              // Setze DB4 auf 1
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_DATA5);              // Setze DB5 auf 1
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_DATA6);              // Setze DB6 auf 0
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_DATA7);              // Setze DB7 auf 0
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);


   // PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_RS);               // Setze RS  auf 0


   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
   asm volatile ("nop");
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
 
   _delay_us(4100);

   // --- Der vorhergehende Befehl wir wiederholt gesendet ---
   // --- Enable-Puls fr Datenbername erzeugen ---
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
   asm volatile ("nop");
   PCF8574_Port_Byte &= ~ (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);


   _delay_us(100);         // --- Warte 100 s; BusyFlag kann noch nicht geprft werden ---


   // --- Der vorhergehende Befehl wir zum dritten Mal gesendet ---
   // --- Enable-Puls fr Datenbername erzeugen ---
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
   asm volatile ("nop");
   PCF8574_Port_Byte &= ~ (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);

   _delay_us(100);         // --- Warte 100 s; BusyFlag kann noch nicht geprft werden ---

   // Sende Kommando 4 Bit Modus zu aktivieren
   // Function set 0x02;
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_DATA4);              // Setze DB4 auf 0
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_DATA5);              // Setze DB5 auf 1
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_DATA6);              // Setze DB6 auf 0
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_DATA7);              // Setze DB7 auf 0
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);

   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
   asm volatile ("nop");
   PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_E);
   Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);

   _delay_us(1000);

   WriteDisplayControl(0x28);                                           // 4Bit / 2 Zeilen / 5x7
   WriteDisplayControl(0x0C);                                           // Display ein / Cursor aus / kein Blinken
   WriteDisplayControl(0x06);                                           // Speicher inkrement / kein Scrollen
 
   ClearDisplay();

   return;
}


/* -------------------------------------------------------------------------------------
   Funktion :         Sendet einen Befehl bzw Daten im 4-Bit-Modus an das Steuer- oder
                      Datenregister des Displays entsprechend der Funktion WriteDisplayControl()
                      oder WriteDisplayData() die diese Funktion aufruft.
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplay(uint8_t data)
{

    I2C_Start();                                             // Erzeugt die Start-Bedingung fr den I2C-Bus
    I2C_Transmit(DEVICE_ADDR+I2C_WRITE_BIT);                 // Sendet die Device-Adresse und das Write-Bit an alle Bausteine
    I2C_Ack();                                               // Wartet auf das Acknowledge-Signal des adressierten Bausteines
 
    // lscht zuerst die Daten die an den Portpins fr das Display anliegen und
    // schiebt dann die oberen vier Bits von Data nach unten (auf PB0 bis PB3)
    // und sendet sie an das Display 
    PCF8574_Port_Byte &= ~((1<<I2C_PORT_BIT_DISPLAY_DATA4) | (1<<I2C_PORT_BIT_DISPLAY_DATA5) | (1<<I2C_PORT_BIT_DISPLAY_DATA6) | (1<<I2C_PORT_BIT_DISPLAY_DATA7));					
    PCF8574_Port_Byte |= (data>>4);
    I2C_Transmit(PCF8574_Port_Byte);                         // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                               // Wartet auf das Acknowledge-Signal des adressierten Bausteines
    // --- Enable-Puls fr Datenbername erzeugen ---
    PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
    I2C_Transmit(PCF8574_Port_Byte);                         // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                               // Wartet auf das Acknowledge-Signal des adressierten Bausteines

    PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_E);
    I2C_Transmit(PCF8574_Port_Byte);                         // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                               // Wartet auf das Acknowledge-Signal des adressierten Bausteines

    // lscht zuerst die Daten die an den Portpins fr das Display anliegen und
    // lscht danach die oberen vier Bits von Data und sendet dann die unteren vier
    // Bits von Data (auf PB0 bis PB3) an das Display 
    PCF8574_Port_Byte &= ~((1<<I2C_PORT_BIT_DISPLAY_DATA4) | (1<<I2C_PORT_BIT_DISPLAY_DATA5) | (1<<I2C_PORT_BIT_DISPLAY_DATA6) | (1<<I2C_PORT_BIT_DISPLAY_DATA7));					
    PCF8574_Port_Byte |= (data & 0x0F);
    I2C_Transmit(PCF8574_Port_Byte);                        // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                              // Wartet auf das Acknowledge-Signal des adressierten Bausteines

    // --- Enable-Puls fr Datenbername erzeugen ---
    PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_E);
    I2C_Transmit(PCF8574_Port_Byte);                        // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                              // Wartet auf das Acknowledge-Signal des adressierten Bausteines

    PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_E);
    I2C_Transmit(PCF8574_Port_Byte);                        // Sendet Daten an den adressierten I2C-Baustein
    I2C_Ack();                                              // Wartet auf das Acknowledge-Signal des adressierten Bausteines

    I2C_Stop();

    return;
}


/* -------------------------------------------------------------------------------------
   Funktion :         Sendet einen Befehl im 4-Bit-Modus an das Steuerregister des Displays
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayControl(uint8_t data)
{
    PCF8574_Port_Byte &= ~(1<<I2C_PORT_BIT_DISPLAY_RS);                       // RS auf 0 setzen
    Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);

    WriteDisplay(data);

    return;
}


/* -------------------------------------------------------------------------------------
   Funktion :         Sendet Daten im 4-Bit-Modus an das Datenregister des Displays
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayData(uint8_t data)
{
    PCF8574_Port_Byte |=  (1<<I2C_PORT_BIT_DISPLAY_RS);                      // RS auf 1 setzen
    Write_I2C(DEVICE_ADDR, PCF8574_Port_Byte);
 
    WriteDisplay(data);

    return;
}


/* -------------------------------------------------------------------------------------
   Funktion :         Diese Routine lscht den am Display angezeigten Text 
   Parameterbergabe: Input     : - 
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void ClearDisplay(void)
{
    WriteDisplayControl(CLEAR_DISPLAY);

    _delay_us(2500); 

    return;
}


/* -------------------------------------------------------------------------------------
   Funktion :         Diese Routine positioniert den Cursor in der Zeile y (1..2) und
                      Spalte x (0..8) 
   Parameterbergabe: Input     : y           Zeile  (1..2)
                                  x           Spalte (0..8)
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void SetDisplayCursor(uint8_t y, uint8_t x)
{
    uint8_t tmp;

    if(y==1)
        tmp=0x80+0x00+x;     // 1. Zeile
    else 
        tmp=0x80+0x40+x;     // 2. Zeile
  
    WriteDisplayControl(tmp);

    return;
}
 

/* -------------------------------------------------------------------------------------
   Funktion :         Diese Routine sendet einen String an das Dsipay
   Parameterbergabe: Input     : *data       der zu sendende Text
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayString(char *data)
{
    while(*data)
    {
        WriteDisplayData(*data);
        data++;
    }

    return;
}
